/*********************************************************************
  All ADuC7023 library code provided by ADI, including this file, is provided
 as is without warranty of any kind, either expressed or implied.
 You assume any and all risk from the use of this code.  It is the
 responsibility of the person integrating this code into an application
 to ensure that the resulting application performs as required and is safe.

 Author        : ADI - Apps            www.analog.com/ADuC7023

 Date          : February 2009

 File          : DAC.c

 Hardware      : Applicable to ADuC7023 rev A & later silicon
                 
 Description   : Demonstartes how to configure each DAC channel
 				DAC0 will output a Sine wave 0 - Vref in amplitude 
				DAC1 will output a Sine wave 0 - AVdd in amplitude 
				DAC2 will toggle between zero scale and full scale
				DAC3 will output a Sine wave 0 - AVdd in amplitude with the output buffer off 
*********************************************************************/
#include <aduc7023.h>
volatile unsigned char ucUpdateDAC2 = 0;
volatile unsigned int uiNewVal = 0;
int main(void) 
{
	// Table is placed in Flash/EE
	const static unsigned short TableS[64] = 
	{
 	0x07FF, 0x08C8, 0x098E, 0x0A51, 0x0B0F, 0x0BC4, 0x0C71, 0x0D12,
 	0x0DA7, 0x0E2E, 0x0EA5, 0x0F0D, 0x0F63, 0x0FA6, 0x0FD7, 0x0FF5,
 	0x0FFF, 0x0FF5, 0x0FD7, 0x0FA6, 0x0F63, 0x0F0D, 0x0EA5, 0x0E2E,
 	0x0DA7, 0x0D12, 0x0C71, 0x0BC4, 0x0B0F, 0x0A51, 0x098E, 0x08C8,
 	0x07FF, 0x0736, 0x0670, 0x05AD, 0x04EF, 0x043A, 0x038D, 0x02EC,
 	0x0257, 0x01D0, 0x0159, 0x00F1, 0x009B, 0x0058, 0x0027, 0x0009,
 	0x0000, 0x0009, 0x0027, 0x0058, 0x009B, 0x00F1, 0x0159, 0x01D0,
 	0x0257, 0x02EC, 0x038D, 0x043A, 0x04EF, 0x05AD, 0x0670, 0x0736  
	};
	unsigned char i = 0;

	POWKEY1 = 0x01;					// Configure CPU Clock for 41.78MHz, CD=0
 	POWCON = 0x00;
 	POWKEY2 = 0xF4;	

	REFCON = 0x01;					// internal 2.5V reference. 2.5V on Vref pin
	DAC0CON = 0x12;					// Set DAC0 output to 0-Vref range and turn DAC0 on, Updated by HCLK.	
	DAC1CON = 0x13;					// Set DAC1 output to 0-AVDD range and turn DAC1 on, Updated by HCLK.	
	DAC2CON = 0x33;					// Set DAC2 output to 0-AVDD range and turn DAC2 on, Updated by Timer1.	
	DAC3CON = 0x53;					// Set DAC3 output to 0-AVDD range and turn DAC3 on, Updated by HCLK, output buffer bypassed
	
	// Configure Timer 1
   T1LD            = 0x700;      	// Enable Timer 1
   T1CON           = 0x4C8;         // Periodic, HCLK, div/256
   IRQEN 		   |=0x8;			// Enable timer1 IRQ source 
	while (1) 
	{
		DAC0DAT = (TableS[i] << 16);
		DAC1DAT = (TableS[i] << 16);
		DAC3DAT = (TableS[i] << 16);  
		i++;
		i &= 0x03f;
		if (ucUpdateDAC2 == 1)
		{
			DAC2DAT = (uiNewVal << 16);	
			ucUpdateDAC2 = 0;
		}
	}								
}

void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	
	IRQSTATUS = IRQSTA;	   			// Read off IRQSTA register
	if ((IRQSTATUS & 0x8) == 0x8)	// Timer1 interrupt source
	{
		T1CLRI = 0xFF;				// Clear current timer Interrupt
		uiNewVal ^=0x800;			// Toggle DAC MSB
		ucUpdateDAC2 = 1;
	}
}
